import json
import sys

ERROR = {
    1: "stacks-node not installed. install manually or run `stnm install",
    2: "stacks-node process is not running",
    3: "stacks-node is already running",
    4: "could not stopped node process",
    5: "not a valid config entry '{}' example: node.miner=true",
    6: "'{}' doesn't exits in available parameters",
    7: "validation failed with message: '{}'",
}


def error_response(code: int, *arg):
    message = ERROR[code]

    resp = {
        "error": True,
        "message": message.format(*arg)
    }

    print(json.dumps(resp))
    sys.exit(1)


SUCCESS = {
    1: "stacks-node process is running",
    2: "stacks-node process killed",
    3: "stacks-node process started",
    4: "config file updated"
}


def success_response(code: int, **kwargs):
    message = SUCCESS[code]

    resp = {
        **{"success": True, "message": message},
        **kwargs
    }

    print(json.dumps(resp))
    sys.exit(0)



import os
import shutil
import subprocess
import sys

from stnm.constants import DEV_NULL, HOME_PATH, CARGO_BIN_PATH
from stnm.shell import which, env

GREEN_COLOR = "\033[92m"
RED_COLOR = "\033[91m"
GRAY_COLOR = "\033[37m"
END_COLOR = "\033[0m"


def success(s: str):
    print("{}stnm: {}{}".format(GREEN_COLOR, s, END_COLOR))


def error(s: str):
    print("{}stnm: {}{}".format(RED_COLOR, s, END_COLOR))
    sys.exit(1)


def info(s: str):
    print("{}stnm: {}{}".format(GRAY_COLOR, s, END_COLOR))


def run_cmd(cmd: str) -> subprocess.CompletedProcess:
    return subprocess.run(cmd, shell=True, env=env, stdout=DEV_NULL, stderr=subprocess.PIPE)


def install():
    if which("stacks-node") is not None:
        success("stacks-node already installed")
        return

    if which("apt-get") is not None:
        info("install dependencies for linux...")
        cmd = "apt install build-essential cmake libssl-dev pkg-config git curl -y"
        if run_cmd(cmd).returncode != 0:
            error("an error occurred")
        else:
            success("done")

    if which("git") is None:
        error("git is not found on your system. please install it first")
        return

    if which("curl") is None:
        error("curl is not found on your system. please install it first")
        return

    info("checking rust...")
    if run_cmd("rustc --version").returncode != 0:
        info("installing rust...")
        cmd = "curl --proto '=https' --tlsv1.2 -sSf https://sh.rustup.rs | sh -s -- -y"
        if run_cmd(cmd).returncode != 0:
            error("an error occurred")
        else:
            success("done")
    else:
        success("rust is already installed")

    assert which("cargo") is not None

    chain_dir = os.path.join(HOME_PATH, "stacks-blockchain-stnm")
    if os.path.isdir(chain_dir):
        shutil.rmtree(chain_dir)

    cmd = "git clone https://github.com/blockstack/stacks-blockchain.git {}".format(chain_dir)
    info("downloading stacks-blockchain...")
    if run_cmd(cmd).returncode != 0:
        error("an error occurred")
    else:
        success("done")

    os.chdir(chain_dir)

    cmd = "cargo build --workspace --release --bin stacks-node"
    info("building stacks-blockchain...")
    if run_cmd(cmd).returncode != 0:
        error("an error occurred")
    else:
        success("done")

    cmd = "cp target/release/stacks-node {}".format(CARGO_BIN_PATH)
    info("copying stacks-node executable to cargo directory...")
    if run_cmd(cmd).returncode != 0:
        error("an error occurred")
    else:
        success("done")

    # clean up
    shutil.rmtree(chain_dir)

    assert which("stacks-node") is not None

    success("installation completed successfully 🎉")



import subprocess

from stnm.config import get_config_path, get_config_parsed, put_config_parsed
from stnm.constants import DEV_NULL
from stnm.response import error_response, success_response
from stnm.shell import env
from stnm.shell import get_node_process


def start():
    process = get_node_process()
    if process is not None:
        error_response(3)

    config = get_config_parsed()
    if "__modified__" in config:
        del config["__modified__"]
        put_config_parsed(config)

    process = subprocess.Popen(["stacks-node", "start", "--config", get_config_path()], env=env, stdout=DEV_NULL, stderr=DEV_NULL)
    success_response(3, **{"pid": process.pid})



from stnm.response import error_response, success_response
from stnm.shell import get_node_process


def stop():
    process = get_node_process()
    if process is None:
        error_response(2)

    try:
        process.kill()
        success_response(2)
    except Exception:
        error_response(4)



import json
import os
from typing import Dict

from flask import Flask, jsonify, request, abort, render_template

from stnm.config import get_config_parsed, get_config, AVAILABLE_PARAMS
from stnm.shell import which

app = Flask(__name__)

this_dir = os.path.abspath(os.path.dirname(__file__))


def communicate(cmd: str, arg: str = "") -> Dict:
    executable = which("stnm")
    out = os.popen("{} {} {} &".format(executable, cmd, arg)).read()
    return json.loads(out)


@app.route("/api", methods=["GET"])
def api_index():
    return jsonify({"hello": "world"})


@app.route("/api/status", methods=["GET"])
def api_status():
    return jsonify(communicate("status"))


@app.route("/api/start", methods=["POST"])
def api_start():
    return jsonify(communicate("start"))


@app.route("/api/stop", methods=["POST"])
def api_stop():
    return jsonify(communicate("stop"))


@app.route("/api/config", methods=["GET"])
def api_config_get():
    config = get_config().strip()
    config_parsed = get_config_parsed()

    resp = {
        "raw": config,
        "object": config_parsed
    }

    return jsonify(resp)


@app.route("/api/config", methods=["POST"])
def api_config_post():
    data = request.json

    if type(data) == dict and "input" in data and type(data["input"]) == str and len(data["input"]) > 0:
        return jsonify(communicate("config", data["input"]))

    abort(400)


@app.route("/api/config-params", methods=["GET"])
def api_config_params():
    return jsonify(list(AVAILABLE_PARAMS.keys()))


@app.route("/ui", methods=["GET"])
def ui_index():
    return render_template("index.jinja2")



import os
import shutil
import subprocess
from typing import Optional

import psutil

from stnm.constants import CARGO_BIN_PATH

env = os.environ.copy()
env["PATH"] = CARGO_BIN_PATH + os.path.pathsep + env["PATH"]


def which(name: str) -> Optional[str]:
    return shutil.which(name, path=env["PATH"])


def get_node_process() -> Optional[psutil.Process]:
    try:
        pid = subprocess.check_output(["pgrep", "-f", "stacks-node"]).decode()
    except subprocess.CalledProcessError:
        return None

    return psutil.Process(int(pid))
